/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ActionFrame.h"
#include "BaseIconNames.h"
#include "CppUtil.h"
#include "Debug.h"
#include "File.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "Singleton.h"
#include "WarningDialog.h"
#include "XmlOptions.h"

#include <QFile>
#include <QShortcut>
#include <QTextStream>

//_______________________________________________
ActionFrame::ActionFrame( QWidget* parent ):
    QGroupBox( parent ),
    Counter( "ActionFrame" )
{

    Debug::Throw( "ActionFrame::ActrionFrame.\n" );
    setLayout( new QVBoxLayout );
    layout()->setMargin( 5 );

    // create checkboxes
    box_ = new QWidget( this );
    layout()->addWidget( box_ );

    QVBoxLayout* checkboxLayout( new QVBoxLayout );
    checkboxLayout->setMargin( 0 );
    box_->setLayout( checkboxLayout );

    // create checkboxes
    checkboxes_ = Base::makeT<CheckBoxMap>( {
        { Latex::Command::Type::Latex, new QCheckBox( tr( "Latex" ), this ) },
        { Latex::Command::Type::PDFLatex, new QCheckBox( tr( "PdfLatex" ), this ) },
        { Latex::Command::Type::Bibtex, new QCheckBox( tr( "Bibtex" ), this ) },
        { Latex::Command::Type::DviPS, new QCheckBox( tr( "Dvi to PS" ), this ) },
        { Latex::Command::Type::DviPDF, new QCheckBox( tr( "Dvi to PDF" ), this ) },
        { Latex::Command::Type::PS2PDF, new QCheckBox( tr( "PS to PDF" ), this ) },
        { Latex::Command::Type::Check, new QCheckBox( tr( "Check" ), this ) },
        { Latex::Command::Type::Clean, new QCheckBox( tr( "Clean" ), this ) }
    } );

    // apply state from options
    int actionState( XmlOptions::get().get<int>( "ACTIONS" ) );
    for( auto&& iter = checkboxes_.begin(); iter != checkboxes_.end(); ++iter )
    {
        checkboxLayout->addWidget( iter.value() );
        if( actionState & (1<<Base::toIntegralType(iter.key())) ) iter.value()->setChecked( true );
        connect( iter.value(), SIGNAL(toggled(bool)), SLOT(_updateButton(bool)) );
    }

    // run button
    button_ = new QPushButton( tr( "Run" ), this );
    button_->setIcon( IconEngine::get( IconNames::DialogAccept ) );
    layout()->addWidget( button_ );
    button_->setToolTip( tr( "Run/abort selected actions" ) );
    connect( button_, SIGNAL(clicked()), SLOT(buttonPressed()) );
    state_ = StateIdle;

    // close accelerator
    new QShortcut( Qt::CTRL + Qt::Key_C, this, SLOT(abort()) );
    new QShortcut( Qt::Key_Return, this, SLOT(buttonPressed()) );

    // configuration
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//_______________________________________________
void ActionFrame::setAutoBibtex( bool value )
{
    Debug::Throw( "ActionFrame::setAutoBibtex.\n" );
    autoBibtex_ = value;
    checkboxes_[Latex::Command::Type::Bibtex]->setVisible( !( autoBibtex_  || XmlOptions::get().raw("BIBTEX_COMMAND" ) == "BIBTEX-NOTFOUND" ) );
}

//_______________________________________________
void ActionFrame::setBusy()
{

    Debug::Throw( "ActionFrame::setBusy.\n" );

    // change button text and status
    button_->setText( tr( "Abort" ) );
    button_->setIcon( IconEngine::get( IconNames::DialogCancel ) );

    // disable widgets
    box_->setEnabled( false );

    // save state
    state_ = StateRunning;

}

//_______________________________________________
void ActionFrame::setIdle()
{

    Debug::Throw( "ActionFrame::setIdle.\n" );

    // process warnings
    if( !warnings_.isEmpty() )
    {
        QString buffer;
        if( warnings_.size() == 1 ) buffer = tr( "Warning: %1" ).arg( warnings_[0] );
        else buffer = tr( "Warning:\n%1" ).arg( warnings_.join( "\n" ) );
        WarningDialog dialog( this, buffer );
        dialog.setDetails( warningLog_ );

        warnings_.clear();
        warningLog_.clear();
        dialog.exec();

    }

    // change button text and status
    button_->setText( tr( "Run" ) );
    button_->setIcon( IconEngine::get( IconNames::DialogAccept ) );

    // enable widgets
    box_->setEnabled( true );

    // save state
    state_ = StateIdle;

}

//_______________________________________________
void ActionFrame::buttonPressed()
{

    if( state_ == StateRunning ) emit commandsAborted();
    else {

        // clear warnings
        warnings_.clear();
        warningLog_.clear();

        // list of latex commands
        Latex::Command::List commands;

        // add a clean call prior to running anything, if selected
        /* this can recover dirty states from previous failed attempts */
        if( _isChecked( Latex::Command::Type::Clean ) ) commands.append(Latex::Command( Latex::Command::Type::Clean, {file_} ) );

        // fill actions depending on CheckBoxes
        // latex
        if( _isChecked( Latex::Command::Type::Latex ) )
        {

            emit recentFileAdded( file_ );
            commands.append( Latex::Command( Latex::Command::Type::Latex, {file_} ) );

            emit cyclesAvailable( 1 );

        }

        // pdf latex
        if( _isChecked( Latex::Command::Type::PDFLatex ) ) {

            emit recentFileAdded( file_ );
            commands.append( Latex::Command( Latex::Command::Type::PDFLatex, {file_} ) );

            emit cyclesAvailable( 1 );

        }

        // bibtex
        const bool needBibtex( !bibtexFile_.isEmpty() && bibtexFile_.exists() );
        if(
            ( autoBibtex_ && needBibtex ) ||
            ( _isChecked( Latex::Command::Type::Bibtex ) && !autoBibtex_ ) )
        {
            // need to run bibtex 3 time to have references right
            commands.append( Latex::Command( Latex::Command::Type::Bibtex, {file_} ) );
            commands.append( Latex::Command( Latex::Command::Type::Bibtex, {file_} ) );
            commands.append( Latex::Command( Latex::Command::Type::Bibtex, {file_} ) );

            if( _isChecked( Latex::Command::Type::Latex ) )
            {

                commands.append( Latex::Command( Latex::Command::Type::Latex, {file_} ) );
                commands.append( Latex::Command( Latex::Command::Type::Latex, {file_} ) );
                emit cyclesAvailable( 3 );

            } else if( _isChecked( Latex::Command::Type::PDFLatex ) ) {

                commands.append( Latex::Command( Latex::Command::Type::PDFLatex, {file_} ) );
                commands.append( Latex::Command( Latex::Command::Type::PDFLatex, {file_} ) );
                emit cyclesAvailable( 3 );

            }

        } else if( _isChecked( Latex::Command::Type::Latex ) ) {

            // run latex a second time
            commands.append( Latex::Command( Latex::Command::Type::Latex, {file_} ) );
            emit cyclesAvailable( 2 );

        } else if( _isChecked( Latex::Command::Type::PDFLatex ) ) {

            // run latex a second time
            commands.append( Latex::Command( Latex::Command::Type::PDFLatex, {file_} ) );
            emit cyclesAvailable( 2 );

        }

        // dvips
        if( _isChecked( Latex::Command::Type::DviPS ) ) commands.append( Latex::Command( Latex::Command::Type::DviPS, {file_} ) );

        // dvipdf
        if( _isChecked( Latex::Command::Type::DviPDF ) ) commands.append( Latex::Command( Latex::Command::Type::DviPDF, {file_} ) );

        // ps2pdf
        if( _isChecked( Latex::Command::Type::PS2PDF ) ) commands.append( Latex::Command( Latex::Command::Type::PS2PDF, {file_} ) );

        // check
        if( _isChecked( Latex::Command::Type::Check ) )
        {
            commands.append( Latex::Command( Latex::Command::Type::CheckCitations, {file_} ) );
            commands.append( Latex::Command( Latex::Command::Type::CheckCitationOrder, {file_} ) );
            commands.append( Latex::Command( Latex::Command::Type::CheckLabels, {file_} ) );
            commands.append( Latex::Command( Latex::Command::Type::CheckReferences, {file_} ) );
        }

        // clean
        if( _isChecked( Latex::Command::Type::Clean ) ) commands.append( Latex::Command( Latex::Command::Type::Clean, {file_} ) );
        if( !commands.empty() )
        {
            _updateActionState();
            emit commandsRequested( commands );
        }

        else InformationDialog( this, tr( "No action taken." ) ).exec();

    }

    return;

}

//_______________________________________________
void ActionFrame::abort()
{ if( state_ == StateRunning ) emit commandsAborted(); }

//_______________________________________________
void ActionFrame::updateBibtexFile( const File& value )
{ bibtexFile_ = value; }

//_______________________________________________
void ActionFrame::_updateButton( bool value )
{
    if( value ) button_->setEnabled( true );
    else {

        button_->setEnabled(
            std::any_of( checkboxes_.begin(), checkboxes_.end(),
            Base::Functor::UnaryTrue<QAbstractButton, &QAbstractButton::isChecked>() ) );

    }


}

//_______________________________________________
void ActionFrame::processWarning( const QString& warning )
{ warnings_.append( warning ); }

//_______________________________________________
void ActionFrame::appendToLog( const QString& warning )
{ warningLog_ += warning; }

//_______________________________________________
void ActionFrame::_updateActionState()
{

    Debug::Throw( "ActionFrame::_updateActionState.\n" );

    // create checkboxes
    int actionState( 0 );
    for( auto&& iter = checkboxes_.begin(); iter != checkboxes_.end(); ++iter )
    { if( iter.value()->isChecked() ) actionState |= (1<<Base::toIntegralType(iter.key())); }

    XmlOptions::get().set<int>("ACTIONS", actionState );
}

//_______________________________________________
void ActionFrame::_updateConfiguration()
{
    Debug::Throw() << "ActionFrame::_updateConfiguration" << endl;

    checkboxes_[Latex::Command::Type::Latex]->setVisible( !(XmlOptions::get().raw("LATEX_COMMAND" ) == "LATEX-NOTFOUND" ) );
    checkboxes_[Latex::Command::Type::PDFLatex]->setVisible( !(XmlOptions::get().raw("PDF_LATEX" ) == "PDFLATEX-NOTFOUND" ) );
    checkboxes_[Latex::Command::Type::Bibtex]->setVisible( !( autoBibtex_  || XmlOptions::get().raw("BIBTEX_COMMAND" ) == "BIBTEX-NOTFOUND" ) );
    checkboxes_[Latex::Command::Type::DviPS]->setVisible( !(XmlOptions::get().raw("DVI_TO_PS" ) == "DVIPS-NOTFOUND" ) );
    checkboxes_[Latex::Command::Type::DviPDF]->setVisible( !(XmlOptions::get().raw("DVI_TO_PDF" ) == "DVIPDF-NOTFOUND" ) );
    checkboxes_[Latex::Command::Type::PS2PDF]->setVisible( !(XmlOptions::get().raw("PS_TO_PDF" ) == "PS2PDF-NOTFOUND" ) );

}
