/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "ActionFrame.h"
#include "ButtonFrame.h"
#include "Config.h"
#include "ConfigurationDialog.h"
#include "Debug.h"
#include "LatexInterface.h"
#include "MenuBar.h"
#include "MainWindow.h"
#include "QtUtil.h"
#include "QuestionDialog.h"
#include "RecentFilesFrame.h"
#include "RecentFilesMenu.h"
#include "XmlFileList.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QMessageBox>
#include <QPixmap>
#include <QTextStream>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( nullptr, arguments  ),
    Counter( "Application" )
{}

//____________________________________________
bool Application::initApplicationManager()
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( commandLineParser( _arguments() ) );
    for( auto& orphan:parser.orphans() )
    { if( !orphan.isEmpty() ) File::expand( orphan ); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    return BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget()
{
    Debug::Throw( "Application::RealizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // file list
    recentFiles_.reset( new XmlFileList );

    // set icon Pixmap
    mainWindow_ = new MainWindow(0);

    // configuration
    connect( this, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    emit configurationChanged();

    // connections
    connect( &mainWindow_->interface(), SIGNAL(busy()), SLOT(busy()) );
    connect( &mainWindow_->interface(), SIGNAL(ready()), SLOT(idle()) );

    // recent files addition
    connect( &mainWindow_->actionFrame(), SIGNAL(recentFileAdded(File)),  SLOT(addRecentFile(File)));
    connect( &mainWindow_->buttonFrame(), SIGNAL(recentFileAdded(File)),  SLOT(addRecentFile(File)));

    // show main windows
    mainWindow_->centerOnDesktop();
    mainWindow_->show();

    const auto& filenames( commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow_->open( FileRecord( File( filenames.front() ).expanded() ) );
    else mainWindow_->open( recentFiles_->lastValidFile() );

    return true;

}

//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );
    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( CommandLineParser::Tag( "--force", "-f" ), tr( "unconditionally open new file in existing window") );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage() const
{
    _usage( "textool", tr( "[options] [file]" ) );
    commandLineParser().usage();
    return;
}

//_________________________________________________
void Application::addRecentFile( File file )
{
    // try add tex extension
    auto texFile( TexFile( file.truncatedName() ).addExtension( ".tex" ) );
    if( !texFile.exists() ) texFile = TexFile( file );

    // update recent file and select
    recentFiles_->add( texFile );
    mainWindow_->menuBar().recentFilesMenu().setCurrentFile( texFile );
    mainWindow_->recentFilesFrame().select( texFile );

}

//_________________________________________________
void Application::_configuration()
{

    Debug::Throw( "Application::configuration.\n" );
    emit saveConfiguration();

    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//_________________________________________________
void Application::_updateConfiguration()
{
    Debug::Throw( "Application::_updateConfiguration.\n" );
    static_cast<XmlFileList*>(recentFiles_.get())->setDBFile( File( XmlOptions::get().raw( "RC_FILE" ) ) );
    recentFiles_->setMaxSize( XmlOptions::get().get<int>( "DB_SIZE" ) );
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow_->uniconify();

        const CommandLineParser parser( commandLineParser( command.arguments() ) );
        const auto filenames( parser.orphans() );
        if( filenames.isEmpty() ) return true;
        else if( parser.hasFlag( "--force" ) ) mainWindow_->open( FileRecord( File( filenames.front() ) ) );
        else {
            const QString buffer = tr( "Accept request for file '%1' ?" ).arg( filenames.front() );
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow_->open( FileRecord( File( filenames.front() ) ) ); }
        }

        return true;

    } else return false;

}
