/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ButtonFrame.h"

#include "Command.h"
#include "Config.h"
#include "LineEditor.h"
#include "Debug.h"
#include "File.h"
#include "InformationDialog.h"
#include "RecentFilesMenu.h"
#include "MainWindow.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QLayout>
#include <QPushButton>
#include <QToolTip>

//_______________________________________________________
ButtonFrame::ButtonFrame( QWidget *parent ):
    QWidget( parent ),
    Counter( "ButtonFrame" )
{

    Debug::Throw( "ButtonFrame::ButtonFrame" );
    QVBoxLayout* layout( new QVBoxLayout );
    setLayout( layout );
    layout->setMargin( 0 );

    // edit tex file
    layout->addWidget( editButton_ = new QPushButton( tr( "Edit Tex" ), this ) );
    editButton_->setToolTip( tr( "Edit tex file" ) );

    // XDVI
    layout->addWidget( viewDviButton_ = new QPushButton( tr( "View DVI" ), this ) );
    connect( viewDviButton_, SIGNAL(clicked()), SLOT(_viewDvi()) );
    viewDviButton_->setToolTip( tr( "View dvi file" ) );

    // View PS
    layout->addWidget( viewPostscriptButton_ = new QPushButton( tr( "View PS" ), this ) );
    connect( viewPostscriptButton_, SIGNAL(clicked()), SLOT(_viewPostscript()) );
    viewPostscriptButton_->setToolTip( tr( "View postscript file" ) );

    // View PDF
    layout->addWidget( viewPdfButton_ = new QPushButton( tr( "View PDF" ), this ) );
    connect( viewPdfButton_, SIGNAL(clicked()), SLOT(_viewPdf()) );
    viewPdfButton_->setToolTip( tr( "View pdf file" ) );

    // clean
    layout->addWidget( cleanButton_ = new QPushButton( tr( "Clean" ), this ) );
    connect( cleanButton_, SIGNAL(clicked()), SLOT(_clean()) );
    cleanButton_->setToolTip( tr( "Delete latex temporary files" ) );

    layout->addStretch(1);

}

//_____________________________________
void ButtonFrame::updateButtons()
{
    Debug::Throw( "ButtonFrame::updateButtons.\n" );
    editButton_->setVisible( file_.addExtension( ".tex" ).exists() );
    viewDviButton_->setVisible( XmlOptions::get().contains( "DVI_VIEWER" ) && file_.addExtension( ".dvi" ).exists() );
    viewPostscriptButton_->setVisible( XmlOptions::get().contains( "PS_VIEWER" ) && file_.addExtension( ".ps" ).exists() );
    viewPdfButton_->setVisible( XmlOptions::get().contains( "PDF_VIEWER" ) && file_.addExtension( ".pdf" ).exists() );

    const auto extensions( TexFile::cleanExtensions() );
    cleanButton_->setVisible( std::any_of( extensions.begin(), extensions.end(),
        [this]( const QString& extension )
        { return file_.addExtension( QString( '.' ) + extension ).exists(); } ) );

}

//_____________________________________
void ButtonFrame::_viewDvi()
{
    Debug::Throw( "ButtonFrame::_viewDvi.\n" );

    // check latex command
    if( !XmlOptions::get().contains( "DVI_VIEWER" ) )
    {
        InformationDialog( this, tr( "No dvi viewer set." ) ).exec();
        return;
    }

    Base::Command dviViewer( XmlOptions::get().raw( "DVI_VIEWER" ) );

    // create/check dvi filename
    auto file( file_.addExtension( ".dvi" ) );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    emit messageAvailable( tr( "Running %1 ..." ).arg( dviViewer.get().join( " " ) ) );

    // run command
    ( dviViewer << file ).run( file.path() );

    emit messageAvailable( QString() );
    emit recentFileAdded( file );
    mainWindow.setFile( file );

    return;
}

//_____________________________________
void ButtonFrame::_viewPostscript()
{
    Debug::Throw( "ButtonFrame::_viewPostscript.\n" );

    // check psViewer command
    if( !XmlOptions::get().contains( "PS_VIEWER" ) ) {
        InformationDialog( this, tr( "PS viewer not set." ) ).exec();
        return;
    }

    Base::Command psViewer( XmlOptions::get().raw("PS_VIEWER") );

    // create/check ps filename
    auto file( file_ .addExtension( ".ps" ) );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    emit messageAvailable( tr( "Running %1..." ).arg( psViewer.get().join( " " ) ) );

    ( psViewer << file ).run( file.path() );

    emit messageAvailable( QString() );
    emit recentFileAdded( file );
    mainWindow.setFile( file );

    return;
}

//_____________________________________
void ButtonFrame::_viewPdf()
{
    Debug::Throw( "ButtonFrame::_RunPDFViewer.\n" );

    // create/check pdf filename
    auto file( file_.addExtension( ".pdf" ) );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    Base::Command pdfViewer( XmlOptions::get().raw( "PDF_VIEWER" ) );
    QString buffer;
    QTextStream( &buffer ) << "Running " << pdfViewer.get().join(" ") << "...";
    emit messageAvailable( tr( "Running %1..." ).arg( pdfViewer.get().join( " " ) ) );

    ( pdfViewer << file.localName() ).run( file.path() );

    emit messageAvailable( QString() );
    emit recentFileAdded( file );
    mainWindow.setFile( file );

    return;
}
