/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "FigureModel.h"

#include "MimeTypeIconProvider.h"

#include <QUrl>

#include <numeric>

//______________________________________________________________________
FigureModel::FigureModel( QObject* parent ):
  FileRecordModel( parent ),
  mimeTypeIconProvider_( new MimeTypeIconProvider( this ) )
{}

//______________________________________________________________________
File FigureModel::fileName( const QModelIndex& index ) const
{

    static std::array<QString,5> extensions = {{ "pdf", "eps", "png", "jpg", "gif" }};

    if( !index.isValid() ) return File();

    const FileRecord record( get(index) );
    File file( record.file() );
    if( file.exists() ) return file;
    else {

        File local;
        return std::any_of( extensions.begin(), extensions.end(),
            [&file, &local]( const QString& extension )
            { return ( local = File( QString( "%1.%2" ).arg( file.truncatedName() ).arg( extension ) ) ).exists(); } ) ?
            local:file;
    }

}

//______________________________________________________________________
File::List FigureModel::fileNames( const QModelIndexList& indexes ) const
{
    File::List files;
    std::transform( indexes.begin(), indexes.end(), std::back_inserter( files ),
        [this] ( const QModelIndex& index ) { return fileName( index ); } );
    return files;
}

//__________________________________________________________________
QVariant FigureModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    if( role == Qt::DecorationRole && index.column() == FileName )
    {

        return mimeTypeIconProvider_->icon( fileName( index ).extension() );

    } else return FileRecordModel::data( index, role );

}

//______________________________________________________________________
QMimeData* FigureModel::mimeData(const QModelIndexList &indexes) const
{

    // get selected filenames
    File::List files( fileNames( indexes ) );

    QOrderedSet<QString> filenames;
    for( const auto& file:files )
    { if( !file.isEmpty() ) filenames.insert( file ); }

    if( filenames.empty() ) return nullptr;
    else {

        auto mimeData = new QMimeData;

        // fill text data
        mimeData->setText(
            std::accumulate( filenames.begin(), filenames.end(), QString(),
            []( const QString& fullText, const QString& filename )
            { return fullText + QString( "file://%1\n" ).arg(filename); } ) );

        // fill url list
        QList<QUrl> urlList;
        std::transform( filenames.begin(), filenames.end(), std::back_inserter( urlList ),
            []( const QString& filename ) { return QString( "file://%1" ).arg(filename); } );
        mimeData->setUrls( urlList );

        return mimeData;

    }

}
