#ifndef LatexCommand_h
#define LatexCommand_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "CppUtil.h"
#include "Debug.h"
#include "File.h"

#include <QString>
#include <QStringList>
#include <QHash>
#include <QList>

//* handles top commands
namespace Latex
{
    class Command final: private Base::Counter<Command>
    {

        public:

        //* enumeration for command type
        enum class Type
        {
            None,
            Latex,
            PDFLatex,
            Bibtex,
            DviPS,
            DviPDF,
            PS2PDF,
            Check,
            CheckCitations,
            CheckCitationOrder,
            CheckLabels,
            CheckReferences,
            FindIncludes,
            FindFigures,
            ToggleIncludeFile,
            Clean
        };

        //* list of commands
        using List = QList<Command>;

        //* constructor
        explicit Command( Type = Type::None, QStringList&& = QStringList() );

        #if QT_VERSION < 0x040800
        //* constructor
        explicit Command( const Type, std::initializer_list<QString>&& );
        #endif

        //*@name accessors
        //@{

        //* command name
        QString name() const
        { return name( type() ); }

        //* map Type to command name
        static QString name( Type type )
        { return _commandNames()[type]; }

        //* id
        int id() const
        { return id_; }

        //* type
        Type type() const
        { return type_; }

        // true if arguments are passed
        bool hasArguments() const
        { return !arguments_.isEmpty(); }

        //* arguments
        const QStringList& arguments() const
        { return arguments_; }

        //* path
        File path() const;

        //@}

        //*@name modifiers
        //@{

        //* add argument
        Command& addArgument( const QString& argument )
        {
            arguments_.append(argument);
            return *this;
        }

        //* clear arguments
        void clearArguments()
        { arguments_.clear(); }

        //@}

        private:

        //* static id counter
        static int& _counter();

        //* map types to command name
        using NameMap = QHash<Type,QString>;

        //* command names
        static const NameMap& _commandNames();

        //* unique id
        int id_ = 0;

        //* type
        Type type_ = Type::None;

        //* arguments
        QStringList arguments_;

    };

    using ::qHash;

    //* less than operator
    inline bool operator < ( const Latex::Command& first, const Latex::Command& second)
    { return first.id() < second.id(); }

    //* equal to operator
    inline bool operator == (const Latex::Command& first, const Latex::Command& second )
    { return first.id() == second.id(); }

}

#endif
