/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "LatexInterface.h"

#include "Command.h"
#include "Debug.h"
#include "TimeStamp.h"
#include "TexFile.h"
#include "TextFormat.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QTextStream>
#include <QRegExp>

namespace Latex
{

    //_______________________________________________
    Interface::Interface( QObject *parent ):
        QObject( parent ),
        Counter( "Interface" )
    { Debug::Throw("Latex::Interface::Interface.\n"); }

    //__________________________________________________________________________________
    void Interface::addCommand( const Command& command )
    {

        Debug::Throw( "Interface::addCommand.\n" );

        // avoid duplicates
        /* this is innefficient */
        const bool empty( commands_.empty() );
        commands_.append( command );

        // emit signal
        emit commandAdded( command );

        // run command if there is none pending
        if( empty )
        {

            // busy signal is disabled for FindIncludes command
            if( command.type() != Command::Type::FindIncludes )
            {
                emit busy();
                emit busyStateChanged( true );
            }

            // run
            _runCommand();
        }

    }

    //__________________________________________________________________________________
    void Interface::addCommands( const Command::List& commands )
    {

        Debug::Throw( "Interface::addCommands.\n" );

        // do nothing if no commands added
        if( commands.empty() ) return;

        // avoid duplicates
        /* this is innefficient */
        const bool empty( commands_.empty() );
        commands_.append( commands );

        emit commandsAdded( commands );
        if( empty )
        {

            // busy signal is disabled for FindIncludes command
            if( commands.front().type() != Command::Type::FindIncludes )
            {
                emit busy();
                emit busyStateChanged( true );
            }

            // run
            _runCommand();
        }

    }

    //__________________________________________________________________________________
    void Interface::abortCommands( bool quiet )
    {
        Debug::Throw( "Interface::abortCommands.\n" );

        if( !quiet ) emit logAvailable( tr( "\nAbort\n" ), Format::Bold );

        // abort process if running
        if( process_ && process_->state() != QProcess::NotRunning )
        {

            process_->kill();
            while( commands_.size() > 1 )
            {
                emit commandCompleted( commands_.back() );
                commands_.removeLast();
            }

        } else {

            // clear current list of commands
            for( const auto& command:commands_ )
            { emit commandCompleted( command ); }
            commands_.clear();

        }

        emit ready();

    }

    //____________________________________________________________
    void Interface::_error( QProcess::ProcessError error )
    {

        QString buffer;
        QTextStream( &buffer ) << CustomProcess::errorMessage( error ) << endl;
        emit logAvailable( "\n" );
        emit logAvailable( buffer, Format::Bold );
        return;

    }

    //______________________________________________________________
    void Interface::_commandCompleted( int code, QProcess::ExitStatus status )
    {

        Q_ASSERT( !commands_.empty() );

        Command command( commands_.front() );
        Debug::Throw() << "Interface::_commandCompleted - command: " << command.name() << " (" << command.id() << ")" << endl;
        if( status != QProcess::NormalExit || ( process_ && process_->error() == QProcess::FailedToStart ) )
        {

            const auto buffer = tr( "Process excited with code %1" ).arg( code );
            emit logAvailable( "\n" );
            emit logAvailable( buffer, Format::Bold|Format::Underline );

            emit commandCompleted( command );
            commands_.removeFirst();

            _runCommand();
            return;

        }

        // process command
        switch( command.type() )
        {
            case Command::Type::Latex:
            case Command::Type::PDFLatex:
            emit maxPageChanged( currentPage_ );
            break;

            default: break;
        }

        // update status
        if( commands_.empty() )
        {
            const auto buffer = tr( "Last update: %1" ).arg( TimeStamp::now().toString( TimeStamp::Format::TimeLong ) );
            emit messageAvailable( buffer );
        }

        // add (cosmetics) empty line to log.
        emit logAvailable( "\n" );

        // emit signal, remove from pile and run next command if any
        emit commandCompleted( command );
        commands_.removeFirst();
        _runCommand();

        return;

    }

    //___________________________________________________________________
    void Interface::_readStdout()
    {
        process_->setReadChannel( QProcess::StandardOutput );
        const QString buffer = QString::fromLocal8Bit( process_->readAll() );
        emit logAvailable( buffer );
        stdoutBuffer_.append( buffer );

        // parse buffer for Latex or PDFLatex command
        if( !commands_.empty() && (
            commands_.front().type() == Command::Type::Latex ||
            commands_.front().type() == Command::Type::PDFLatex ) )
        {
            QRegExp regexp( "\\[(\\d+)" );
            if( buffer.indexOf( regexp ) >= 0 )
            {
                currentPage_ = regexp.cap(1).toInt();
                emit pageChanged( currentPage_ );
            }
        }

    }

    //_____________________________________________________
    bool Interface::_checkFile( File file, const QString& command, bool quiet )
    {

        Debug::Throw() << "Interface::_checkFile - command: " << command << ", quiet: " << quiet << endl;
        file.expand();

        if( !file.exists() )
        {
            if( !quiet )
            {
                QString buffer = QString( "File '%1' does not exists." ).arg( file );
                if( command.isEmpty() ) buffer += tr( " Command aborted." );
                else buffer+= QString( " %1 command aborted." ).arg( command );
                InformationDialog( qApp->activeWindow(), buffer ).exec();
            }
            return false;
        }

        if( file.isDirectory() )
        {
            if( !quiet )
            {
                const QString buffer = tr( "File '%1' is a directory. Aborted." ).arg( file );
                InformationDialog( qApp->activeWindow(), buffer ).exec();
            }
            return false;
        }

        return true;
    }

    //___________________________________________________________________
    void Interface::_readStderr()
    {
        process_->setReadChannel( QProcess::StandardError );
        const QString buffer = QString::fromLocal8Bit( process_->readAll() );
        emit logAvailable( buffer, Format::Italic );
        stderrBuffer_.append( buffer );
    }

    //____________________________________________________________
    void Interface::_resetBuffers()
    {
        Debug::Throw( "Interface::_resetBuffer.\n" );
        stdoutBuffer_.clear();
        stderrBuffer_.clear();
    }

    //__________________________________________________________________________________
    bool Interface::_runCommand()
    {

        // do nothing if no commands are in the pile
        if( commands_.empty() )
        {

            Debug::Throw( "Interface::_runCommand - ready \n" );
            emit ready();
            emit busyStateChanged( false );
            return false;

        }

        // front command
        Command command( commands_.front() );
        Debug::Throw() << "Interface::_runCommand - command: " << command.name() << " (" << command.id() << ")" << endl;

        // reset buffer
        _resetBuffers();
        process_.reset();

        // take action depending on command type
        switch( command.type() )
        {

            case Command::Type::Latex:
            {

                auto file( TexFile( command.path() ).addExtension( ".tex" ) );
                if( !_checkFile( file, "Latex" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running Latex:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running Latex" ) );

                // reset currentPage
                currentPage_ = 0;
                emit pageChanged( currentPage_ );

                // build command and execute
                const auto arguments = Base::makeT<QStringList>({ XmlOptions::get().raw("LATEX_COMMAND" ), file.localName() });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );

                // run and emit signal
                emit commandStarted( command );
                return true;
            }

            case Command::Type::PDFLatex:
            {

                auto file( TexFile( command.path() ).addExtension( ".tex" ) );
                if( !_checkFile( file, "PDFLatex" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running PDFLatex:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running PDFLatex" ) );

                // reset currentPage
                currentPage_ = 0;
                emit pageChanged( currentPage_ );

                // build command and execute
                const auto arguments = Base::makeT<QStringList>({ XmlOptions::get().raw("PDF_LATEX" ), file.localName() });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );

                // run and emit signal
                emit commandStarted( command );
                return true;
            }

            case Command::Type::Bibtex:
            {
                auto file( TexFile( command.path() ).addExtension( ".aux" ) );
                if( !_checkFile( file, "Bibtex" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running Bibtex:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running Bibtex" ) );

                const auto arguments = Base::makeT<QStringList>({ XmlOptions::get().raw("BIBTEX_COMMAND" ), file.truncatedName().localName() });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );
                return true;
            }

            case Command::Type::DviPS:
            {
                auto file( TexFile( command.path() ).addExtension( ".dvi" ) );
                if( !_checkFile( file, "DviPS" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running Dvi to PS:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running Dvi to PS" ) );

                // create ps filename
                auto psFile( file.addExtension( ".ps" ) );
                const auto arguments = Base::makeT<QStringList>({
                    XmlOptions::get().raw("DVI_TO_PS" ),
                        "-o",
                        psFile.localName(),
                        file.truncatedName().localName()
                    });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );
                return true;
            }

            case Command::Type::DviPDF:
            {
                auto file( TexFile( command.path() ).addExtension( ".dvi" ) );
                if( !_checkFile( file, "DviPDF" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running Dvi do PDF:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running Dvi to PDF" ) );

                // create ps filename
                auto pdfFile( file.addExtension( ".pdf" ) );
                const auto arguments = Base::makeT<QStringList> ({
                    XmlOptions::get().raw("DVI_TO_PDF" ),
                        file.truncatedName().localName(),
                        pdfFile.localName()
                    });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );
                return true;
            }

            case Command::Type::PS2PDF:
            {
                auto file( TexFile( command.path() ).addExtension( ".dvi" ) );
                if( !_checkFile( file, "PS to PDF" ) )
                {
                    abortCommands( true );
                    break;
                }

                emit logAvailable( tr( "Running PS to PDF:\n" ), Format::Bold|Format::Underline );
                emit messageAvailable( tr( "Running PS to PDF" ) );

                // create ps filename
                auto pdfFile( file.addExtension( ".pdf" ) );
                const auto arguments  = Base::makeT<QStringList>({
                    XmlOptions::get().raw("DVI_TO_PDF" ),
                        file.truncatedName().localName(),
                        pdfFile.localName()
                    });

                _newProcess();
                process_->setWorkingDirectory( file.path() );
                process_->start( arguments );
                return true;
            }

            case Command::Type::CheckCitations:
            {
                auto file( TexFile( command.path() ).addExtension( ".log" ) );
                if( _checkFile( file, tr( "Citations check" ) ) )
                {

                    emit messageAvailable( tr( "Checking Citations" ) );
                    _checkCitations( file );

                } else abortCommands( true );
                break;
            }

            case Command::Type::CheckLabels:
            {
                auto file( TexFile( command.path() ).addExtension( ".log" ) );
                if( _checkFile( file, tr( "Label check" ) ) )
                {

                    emit messageAvailable( tr( "Checking Labels" ) );
                    _checkLabels( file );

                } else abortCommands( true );
                break;
            }

            case Command::Type::CheckReferences:
            {
                auto file( TexFile( command.path() ).addExtension( ".log" ) );
                if( _checkFile( file, tr( "References check" ) ) )
                {

                    emit messageAvailable( tr( "Checking References" ) );
                    _checkReferences( file );

                } else abortCommands( true );
                break;
            }

            case Command::Type::CheckCitationOrder:
            {
                auto file( TexFile( command.path() ).addExtension( ".tex" ) );
                if( _checkFile( file, tr( "Citations order check" ) ) )
                {

                    emit messageAvailable( tr( "Checking Citations Order" ) );
                    _checkCitationOrder( file );

                } else abortCommands( true );

                break;
            }

            case Command::Type::FindIncludes:
            {
                // clear
                includeFiles_.clear();
                emit bibtexFileAvailable( File() );

                auto file( TexFile( command.path() ).addExtension( ".tex" ) );
                if( _checkFile( file, "Parsing", true ) )
                {
                    emit messageAvailable( tr( "Checking included files" ) );
                    _findIncludes( IncludeFile( file ), includeFiles_ );

                } else abortCommands( true );

                emit includeFilesAvailable( includeFiles_ );
                break;
            }

            case Command::Type::ToggleIncludeFile:
            {

                const File parent( command.arguments()[0] );
                const File file( command.arguments()[1] );
                if( _checkFile( parent, tr( "Parsing" ) ) ) _toggleIncludeFile( parent, file );
                else abortCommands( true );
                break;

            }

            case Command::Type::FindFigures:
            {
                auto file( TexFile( command.path() ).addExtension( ".tex" ) );
                if( _checkFile( file, tr( "Parsing" ), true ) )  _findFigures( file );
                else abortCommands( true );
                break;
            }

            case Command::Type::Clean:
            {
                emit messageAvailable( tr( "Cleaning" ) );
                bool first = true;
                _clean( command.path(), first );

                // also remove include files aux
                for( const auto& includeFile:includeFiles_ )
                { _clean( includeFile.file(), "aux", first ); }

                if( !first ) emit logAvailable( "\n" );

                break;
            }

            default:
            break;
        }

        emit commandCompleted( command );

        if( !commands_.empty() )
        { commands_.removeFirst(); }

        // process next command
        return _runCommand();

    }

    //____________________________________________________________
    void Interface::_checkReferences( const File& file )
    {

        Debug::Throw() << "Interface::_checkReferences - file: " << file << endl;

        // parse file
        QFile in( file );
        if( !in.open( QIODevice::ReadOnly ) ) return;

        // read all text
        QString body = in.readAll();
        in.close();

        body.replace( "\n", "" );

        // undefined references RegExp
        // prototype: LaTeX Warning: Reference `ssec_v2' on page 16 undefined on input line 25.
        QRegExp regexp( "LaTeX Warning\\:\\s+Reference\\s+\\`(\\S+)\\'\\s+on\\s+page\\s+(\\d+)\\s+undefined\\s+on\\s+input\\s+line\\s+(\\d+)" );
        regexp.setMinimal( true );

        // output buffer
        QString buffer;
        bool found( false );

        for( int position = 0; (position = body.indexOf( regexp, position )) >= 0; position += regexp.matchedLength() )
        {
            buffer += tr( "LaTeX Warning: Reference \'%1' on page %2 is undefined in input line %3.\n" )
                .arg( regexp.cap(1), regexp.cap(2),regexp.cap(3) );
            found = true;
        }

        // return if none found
        if( found )
        {
            emit warningAvailable( tr( "There are undefined references." ) );
            emit warningLogAvailable( tr( "Undefined references:\n" ), Format::Bold|Format::Underline );
            emit warningLogAvailable( buffer );
            emit warningLogAvailable( "\n" );
        }

        return;

    }

    //____________________________________________________________
    void Interface::_checkCitations( const File& file )
    {

        Debug::Throw() << "Interface::_checkCitations - file: " << file << endl;

        // parse file
        QFile in( file );
        if( !in.open( QIODevice::ReadOnly ) ) return;

        // read all text
        QString body = in.readAll();
        in.close();

        body.replace( "\n", "" );

        // undefined references RegExp
        // prototype:
        // LaTeX Warning: Citation `ramona' on page 40 undefined on input line 115.
        // Package natbib Warning: Citation `Aaij:2015rlaX' on page 7 undefined on input line 17.

        QRegExp regexp( "((?:LaTeX)|(?:Package natbib)\\s+Warning)\\:\\s+Citation\\s+\\`(\\S+)\\'\\s+on\\s+page\\s+(\\d+)\\s+undefined\\s+on\\s+input\\s+line\\s+(\\d+)" );
        regexp.setMinimal( true );

        // output buffer
        QString buffer;
        bool found( false );

        for( int position = 0; ( position = body.indexOf( regexp, position ) ) >= 0; position += regexp.matchedLength() )
        {
            buffer += tr( "%1: Citation \'%2' on page %3 is undefined in input line %4.\n" )
                .arg( regexp.cap(1), regexp.cap(2), regexp.cap(3), regexp.cap(4) );
            found = true;
        }

        // return if none found
        if( found )
        {
            emit warningAvailable( tr( "There are undefined citations." ) );
            emit warningLogAvailable( tr( "Undefined citations:\n" ), Format::Bold|Format::Underline );
            emit warningLogAvailable( buffer );
            emit warningLogAvailable( "\n" );
        }

        return;

    }

    //____________________________________________________________
    void Interface::_checkLabels( const File& file )
    {
        Debug::Throw() << "Interface::_checkLabels - file: " << file << endl;

        // parse file
        QFile in( file );
        if( !in.open( QIODevice::ReadOnly ) ) return;

        // read all text
        QString body = in.readAll();
        in.close();

        // undefined references RegExp
        // prototype: LaTeX Warning: Label `fig_phase' multiply defined.
        QRegExp regexp( "LaTeX Warning\\:\\s+Label\\s+\\`(\\S+)\\'\\s+multiply\\s+defined" );
        regexp.setMinimal( true );

        // output buffer
        QString buffer;
        bool found( false );
        for( int position = 0; ( position = body.indexOf( regexp, position ) ) >= 0; position += regexp.matchedLength() )
        {
            buffer += tr( "LaTeX Warning: Label \'%1' multiply defined\n" ).arg( regexp.cap(1) );
            found = true;
        }

        // return if none found
        if( found )
        {
            emit warningAvailable( tr( "There are multiply defined labels." ) );
            emit warningLogAvailable( tr( "Label warnings:\n" ), Format::Bold|Format::Underline );
            emit warningLogAvailable( buffer );
            emit warningLogAvailable( "\n" );
        }

        return;
    }


    //____________________________________________________________
    void Interface::_checkCitationOrder( const File& file )
    {

        Debug::Throw() << "Interface::_checkCitationOrder - file: " << file << endl;

        QRegExp citationRegExp( "\\\\cite\\{+(.*)\\}" );
        citationRegExp.setMinimal( true );

        // loop over include files
        QStringList citationList;
        for( const auto& includeFile:includeFiles_ )
        {

            // check file existence
            if( !checkFile( includeFile.file(), true ) ) continue;

            // try open file
            File fullname( includeFile.file().expanded() );
            QFile in( fullname );
            if( !in.open( QIODevice::ReadOnly ) ) continue;

            // retrieve text
            QString text( in.readAll() );

            // split by lines
            const auto lines( text.split( "\n" ) );

            for( const auto& line:lines )
            {

                // get position of regexp in line.
                for( int position = 0; ( position = citationRegExp.indexIn( line, position ) ) >= 0; position += citationRegExp.matchedLength() )
                {

                    if( !citationRegExp.cap(1).isEmpty() )
                    {

                        // split using "," as a separator in case of multiple citations
                        const auto citations( citationRegExp.cap(1).split( "," ) );
                        for( const auto& citation:citations )
                        { if( !citationList.contains( citation ) ) citationList.append( citation ); }

                    }

                }

            }

        }

        // emit log
        if( !citationList.empty() )
        {
            emit logAvailable( tr( "Citation order:\n" ), Format::Bold|Format::Underline );
            emit logAvailable( citationList.join("\n") );
            emit logAvailable( "\n" );
        }

    }

    //____________________________________________________________
    void Interface::_findIncludes( const IncludeFile& parent, IncludeFile::List& includeFiles )
    {

        includeFiles.append( parent );

        // regular expressions
        QRegExp includeRegexp( "\\\\(input|include|bibliography)\\{(.+)\\}" );
        QRegExp commentRegexp( "^(\\s*)%" );

        includeRegexp.setMinimal( true );

        // try open File
        File fullname( parent.file() );
        QFile in( fullname );
        if( !in.open( QIODevice::ReadOnly ) ) return;

        // retrieve text
        QString text( in.readAll() );
        in.close();
        const auto lines( text.split( "\n" ) );
        for( const auto& line:lines )
        {

            // get position of regexp in line.
            for( int position = 0; ( position = includeRegexp.indexIn( line, position ) ) >= 0; position += includeRegexp.matchedLength() )
            {

                Debug::Throw() << "Interface::_findIncludes - found match in line " << line << endl;

                // get found File
                bool isBibtex( false );
                TexFile includeFile( File( includeRegexp.cap(2) ) );
                if( includeRegexp.cap(1) == "bibliography" )
                {

                    includeFile = includeFile.addExtension( ".bib" );
                    isBibtex = true;

                } else includeFile = includeFile.addExtension( ".tex" );

                if( !includeFile.isAbsolute() ) includeFile.addPath( parent.file().path() );

                // see if enabled
                const bool enabled( commentRegexp.indexIn( line ) < 0 );
                if( isBibtex && enabled ) emit bibtexFileAvailable( includeFile );

                // add File and enabled state to the list
                IncludeFile current = IncludeFile( FileRecord( includeFile ) );
                current.setParent( parent );
                current.setSelected( enabled );

                // check for duplicates
                if( includeFiles.contains( current ) )
                {

                    const QString buffer = tr( "Duplicated File found in include list '%1'\n" ).arg( current.file() );
                    InformationDialog( qApp->activeWindow(), buffer ).exec();

                } else {

                    _findIncludes( current, includeFiles );

                }

            }

        }

        return;

    }

    //____________________________________________________________
    void Interface::_toggleIncludeFile( const File& parent, const File& file ) const
    {
        Debug::Throw() << "Interface::_toggleIncludeFile - parent: " << parent << " file: " << file << endl;

        // regular expressions
        QRegExp includeRegexp( "\\\\(input|include|bibliography)\\{(.+)\\}" );
        QRegExp commentRegexp( "^(\\s*)%" );
        includeRegexp.setMinimal( true );

        // read text
        QFile in( parent );
        if( !in.open( QIODevice::ReadOnly ) ) return;
        QString text( in.readAll() );
        in.close();

        // split by lines
        auto lines( text.split( "\n" ) );

        bool found( false );
        for( auto& line:lines )
        {

            // get position of regexp in line.
            for( int position = 0; ( position = includeRegexp.indexIn( line, position ) ) >= 0; position += includeRegexp.matchedLength() )
            {

                // see if found File match
                TexFile includeFile( File( includeRegexp.cap(2) ) );
                if( includeRegexp.cap(1) == "bibliography" )  includeFile = includeFile.addExtension( ".bib" );
                else includeFile = includeFile.addExtension( ".tex" );

                if( !includeFile.isAbsolute() ) includeFile.addPath( file.path() );
                if( includeFile == file )
                {

                    Debug::Throw() << "LateInterface::_toggleIncludeFile - found File: " << includeFile << endl;

                    // see if line is commented
                    if( commentRegexp.indexIn( line ) >= 0 )
                    {

                        line.replace( commentRegexp, commentRegexp.cap(1) );

                    } else {

                        line = "%"+line;
                    }

                    found = true;
                    break;
                }

            }

            if( found ) break;

        }

        // make a backup of the input File
        if( XmlOptions::get().get<bool>( "SAVE_BACKUP" ) )
        { parent.backup(); }

        // create output stream
        QFile out( parent );
        if( !out.open( QIODevice::WriteOnly ) ) return;

        // write lines
        for( const auto& line:lines )
        { out.write( qPrintable( QString( line+"\n" ) ) ); }

        out.close();

        return;
    }

    //____________________________________________________________
    void Interface::_findFigures( const File& )
    {
        Debug::Throw( "Interface::_findFigures.\n" );

        // regular expressions
        QRegExp graphicsPathRegExp( "\\\\graphicspath\\{+((?:\\w|/|\\.\\.)+)\\}" );
        QRegExp figureRegExp( "\\\\includegraphics(?:\\[.*\\])?\\{+(.+)\\}" );
        QRegExp commentRegExp( "^(\\s*)%" );

        figureRegExp.setMinimal( true );

        // clear graphics path
        File graphicsPath;
        emit graphicsPathAvailable( graphicsPath );

        // list of figures
        FileRecord::List figures;

        // loop over include files
        for( const auto& includeFile:includeFiles_ )
        {

            // try open file
            File fullname( includeFile.file().expanded() );
            QFile in( fullname );
            if( !in.open( QIODevice::ReadOnly ) ) continue;

            // retrieve text
            QString text( in.readAll() );
            in.close();

            // look for graphics tag
            const auto lines( text.split( "\n" ) );
            for( const auto& line:lines )
            {
                // check if match
                if( graphicsPathRegExp.indexIn( line ) < 0 ) continue;

                // check if commented
                if( commentRegExp.indexIn( line ) >= 0 ) continue;

                // store graphic path
                graphicsPath = File( graphicsPathRegExp.cap(1) );
                Debug::Throw() << "Interface::_findFigures - graphicsPath:" << graphicsPath << endl;
                if( !graphicsPath.isAbsolute() ) graphicsPath.addPath( fullname.path() );
                emit graphicsPathAvailable( graphicsPath );
            }

            // parse file for figures
            for( const auto& line:lines )
            {

                for( int position = 0; ( position = figureRegExp.indexIn( line, position ) ) >= 0; position += figureRegExp.matchedLength() )
                {

                    // check if commented
                    if( commentRegExp.indexIn( line ) < 0 )
                    {
                        File figure( figureRegExp.cap(1) );
                        if( !figure.isAbsolute() )
                        {
                            if( !graphicsPath.isEmpty() ) figure.addPath( graphicsPath );
                            else figure.addPath( fullname.path() );
                        }

                        // create new item
                        figures.append( FileRecord( figure ) );

                    }

                }
            }

        }

        // emit signal
        if( !figures.empty() ) emit figuresAvailable( figures );

    }

    //_______________________________________________
    void Interface::_clean( const File& file, bool& first )
    {

        Debug::Throw( "MainWindow::_clean.\n" );
        for( const auto& extension : TexFile::cleanExtensions() )
        { _clean( file, extension, first ); }

        return;
    }

    //_______________________________________________
    void Interface::_clean( const File& file, const QString& extension, bool& first )
    {

        // check option
        const QString optionName = "AUTO_CLEAN_"+extension;
        if( XmlOptions::get().contains( optionName ) && !XmlOptions::get().get<bool>( optionName ) )
        { return; }

        auto local( TexFile( file ).addExtension( QString(".")+extension ) );
        if( local.exists() )
        {

            if( first )
            {
                emit logAvailable( tr( "Cleaning\n" ), Format::Bold|Format::Underline );
                first = false;
            }

            QString buffer;
            QTextStream( &buffer ) << "rm " << local.localName() << "\n";
            emit logAvailable( buffer );
            local.remove();

        }

        return;
    }

    //____________________________________________________________
    CustomProcess& Interface::_newProcess()
    {

        Debug::Throw( "Interface::_newProcess.\n" );

        // create new process and assign customized environment
        process_.reset( new CustomProcess( this ) );
        connect( process_.get(), SIGNAL(error(QProcess::ProcessError)), SLOT(_error(QProcess::ProcessError)) );
        connect( process_.get(), SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(_commandCompleted(int,QProcess::ExitStatus)) );
        connect( process_.get(), SIGNAL(readyReadStandardOutput()), SLOT(_readStdout()) );
        connect( process_.get(), SIGNAL(readyReadStandardError()), SLOT(_readStderr()) );
        return *process_.get();
    }

}
