#ifndef LatexInterface_h
#define LatexInterface_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CustomProcess.h"
#include "Debug.h"
#include "File.h"
#include "IncludeFile.h"
#include "InformationDialog.h"
#include "LatexCommand.h"
#include "MessageBuffer.h"
#include "TextFormat.h"

#include <QObject>
#include <QProcess>
#include <QString>

namespace Latex
{
    class Interface: public QObject, private Base::Counter<Interface>
    {

        //* Qt meta object declaration
        Q_OBJECT

            public:

            //* constructor
            explicit Interface( QObject* = nullptr );

        //* check file existence
        bool checkFile( File file, bool quiet = false )
        { return _checkFile( file, QString(), quiet ); }

        Q_SIGNALS:

        //* command added
        void commandAdded( const Latex::Command& );

        //* commands added
        void commandsAdded( const Latex::Command::List& );

        //* command completed
        void commandCompleted( const Latex::Command& );

        //* command started
        void commandStarted( const Latex::Command& );

        //* emitted when executing a command
        void busy();

        //* emitted when there is no more commands in pile
        void ready();

        //* emitted when busy state changed
        void busyStateChanged( bool );

        //* logAvailable
        void logAvailable( const QString&, Format::TextFormatFlags = Format::Default );

        //* logAvailable
        void warningLogAvailable( const QString&, Format::TextFormatFlags = Format::Default );

        //* warnings
        void warningAvailable( const QString& );

        //* message
        void messageAvailable( const QString& );

        //* include files available
        void includeFilesAvailable( const IncludeFile::List& );

        //* bibliography file
        void bibtexFileAvailable( const File& );

        //* graphics path available
        void graphicsPathAvailable( const File& );

        //* figure list available
        void figuresAvailable( const FileRecord::List& );

        //* current page changed
        void pageChanged( int );

        //* max page change
        void maxPageChanged( int );

        public Q_SLOTS:

        //* add command
        void addCommand( const Latex::Command& );

        //* add commands
        void addCommands( const Latex::Command::List& );

        //* abort command
        void abortCommands( bool quiet = false );

        private Q_SLOTS:

        //* error from process
        void _error( QProcess::ProcessError );

        //* take action when current command completed
        void _commandCompleted( int, QProcess::ExitStatus );

        //* read on-fly from process
        void _readStdout();

        //* read on-fly from process
        void _readStderr();

        private:

        //* check file existence
        bool _checkFile( File, const QString&, bool quiet = false );

        //* reset buffers
        void _resetBuffers();

        //* run next command in pile
        bool _runCommand();

        //* check references
        void _checkReferences( const File& );

        //* check citations
        void _checkCitations( const File& );

        //* check labels
        void _checkLabels( const File& );

        //* check citations order
        void _checkCitationOrder( const File& );

        //* find includes (recursive)
        void _findIncludes( const IncludeFile&, IncludeFile::List& );

        //* toggle include file
        void _toggleIncludeFile( const File&, const File& ) const;

        //* find graphics path
        void _findFigures( const File& );

        //* clean
        void _clean( const File&, bool& first );

        //* clean
        void _clean( const File&, const QString&, bool& first );

        //* create new process to run command
        CustomProcess& _newProcess();

        //* custom process
        CustomProcess::Pointer process_;

        //* stdout buffer
        MessageBuffer stdoutBuffer_;

        //* stderr buffer
        MessageBuffer stderrBuffer_;

        //* map command type to name
        Command::List commands_;

        //* cached list of include files
        IncludeFile::List includeFiles_;

        //* page number
        int currentPage_ = 0;

    };

}

#endif
