/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"

#include "Application.h"
#include "ActionFrame.h"
#include "BaseIconNames.h"
#include "BrowsedLineEditor.h"
#include "ButtonFrame.h"
#include "Command.h"
#include "FileDialog.h"
#include "FileList.h"
#include "FigureFrame.h"
#include "IconEngine.h"
#include "IncludeFrame.h"
#include "InformationDialog.h"
#include "LatexCommandDialog.h"
#include "LatexInterface.h"
#include "LineEditor.h"
#include "LogFrame.h"
#include "LogWidget.h"
#include "MenuBar.h"
#include "StatusBar.h"
#include "RecentFilesFrame.h"
#include "RecentFilesMenu.h"
#include "Singleton.h"
#include "TextFormat.h"
#include "TexFile.h"
#include "TexString.h"
#include "Util.h"

#include <QApplication>

//_______________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" )
{

    _updateWindowTitle();
    setOptionName( "MAIN_WINDOW" );

    // create interface
    interface_ = new Latex::Interface( this );

    Debug::Throw( "MainWindow::MainWindow.\n" );
    auto main = new QWidget( this );
    auto mainLayout = new QVBoxLayout;
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    setCentralWidget( main );

    top_ = new QWidget( main );
    mainLayout->addWidget( top_ );
    QVBoxLayout* topLayout = new QVBoxLayout;
    topLayout->setMargin(0);
    top_->setLayout( topLayout );

    // label and file editor
    auto browsedEditor = new BrowsedLineEditor( top_ );
    editor_ = &browsedEditor->editor();
    topLayout->addWidget( browsedEditor );

    // horizontal box for action frame and button frame
    QHBoxLayout* hLayout = new QHBoxLayout;
    hLayout->setMargin(0);
    topLayout->addLayout( hLayout, 0 );

    auto application( Base::Singleton::get().application<Application>() );
    hLayout->addWidget( actionFrame_ = new ActionFrame( top_ ) );
    hLayout->addWidget( buttonFrame_ = new ButtonFrame( top_ ) );

    // tab widget
    tabWidget_ = new QTabWidget( main );
    mainLayout->addWidget( tabWidget_, 1 );

    // add tabs to widget
    tabWidget_->addTab( logFrame_ = new LogFrame( tabWidget_ ), tr( "Log" ) );
    tabWidget_->addTab( includeFrame_ = new IncludeFrame( tabWidget_ ), tr( "Includes" ) );
    tabWidget_->addTab( figureFrame_ = new FigureFrame( tabWidget_ ), tr( "Figures" ) );
    tabWidget_->addTab( recentFilesFrame_ = new RecentFilesFrame( tabWidget_, application->recentFiles() ), tr( "Recent Files" ) );

    // statusBar
    setStatusBar( statusbar_ = new StatusBar( this ) );

    // actions
    _installActions();
    addAction( &application->closeAction() );

    // menu
    setMenuBar( menuBar_ = new MenuBar( this ) );

    actionFrame_->button().setFocus();

    //* dialogs
    commandDialog_ = new Latex::CommandDialog;
    connect( commandDialog_, SIGNAL(dialogHidden()), SLOT(_commandDialogHidden()) );

    // connections
    connect( editor_, SIGNAL(textChanged(QString)), SLOT(_updateFile()) );

    connect( actionFrame_, SIGNAL(commandRequested(Latex::Command)), interface_, SLOT(addCommand(Latex::Command)) );
    connect( actionFrame_, SIGNAL(commandsRequested(Latex::Command::List)), interface_, SLOT(addCommands(Latex::Command::List)) );
    connect( actionFrame_, SIGNAL(commandsAborted()), interface_, SLOT(abortCommands()) );
    connect( buttonFrame_, SIGNAL(commandRequested(Latex::Command)), interface_, SLOT(addCommand(Latex::Command)) );
    connect( includeFrame_, SIGNAL(commandRequested(Latex::Command)), interface_, SLOT(addCommand(Latex::Command)) );
    connect( includeFrame_, SIGNAL(commandsRequested(Latex::Command::List)), interface_, SLOT(addCommands(Latex::Command::List)) );
    connect( includeFrame_, SIGNAL(messageAvailable(QString)), &statusbar_->label(), SLOT(setText(QString)) );
    connect( buttonFrame_, SIGNAL(messageAvailable(QString)), &statusbar_->label(), SLOT(setText(QString)) );
    connect( figureFrame_, SIGNAL(messageAvailable(QString)), &statusbar_->label(), SLOT(setText(QString)) );
    connect( figureFrame_, SIGNAL(commandRequested(Latex::Command)), interface_, SLOT(addCommand(Latex::Command)) );

    connect( recentFilesFrame_, SIGNAL(fileSelected(FileRecord)), SLOT(open(FileRecord)) );

    connect( interface_, SIGNAL(logAvailable(QString,Format::TextFormatFlags)), &logFrame_->editor(), SLOT(append(QString,Format::TextFormatFlags)) );
    connect( interface_, SIGNAL(warningLogAvailable(QString,Format::TextFormatFlags)), &logFrame_->editor(), SLOT(append(QString,Format::TextFormatFlags)) );

    connect( interface_, SIGNAL(messageAvailable(QString)), &statusbar_->label(), SLOT(setText(QString)) );
    connect( interface_, SIGNAL(commandCompleted(Latex::Command)), SLOT(_processCompletedCommand(Latex::Command)) );
    connect( interface_, SIGNAL(warningAvailable(QString)), actionFrame_, SLOT(processWarning(QString)));
    connect( interface_, SIGNAL(warningLogAvailable(QString,Format::TextFormatFlags)), actionFrame_, SLOT(appendToLog(QString)) );
    connect( interface_, SIGNAL(bibtexFileAvailable(File)), SLOT(_updateBibtexFile(File)) );

    connect( interface_, SIGNAL(busy()), actionFrame_, SLOT(setBusy()) );
    connect( interface_, SIGNAL(busy()), &logFrame_->editor(), SLOT(clear()) );
    connect( interface_, SIGNAL(ready()), actionFrame_, SLOT(setIdle()) );
    connect( interface_, SIGNAL(ready()), &statusbar_->label(), SLOT(clear()) );
    connect( interface_, SIGNAL(busyStateChanged(bool)), editor_, SLOT(setDisabled(bool)) );

    // progress
    connect( interface_, SIGNAL(maxPageChanged(int)), statusbar_, SLOT(setMaximum(int)) );
    connect( interface_, SIGNAL(pageChanged(int)), statusbar_, SLOT(setValue(int)) );
    connect( interface_, SIGNAL(ready()), statusbar_, SLOT(showLabel()) );
    connect( actionFrame_, SIGNAL(cyclesAvailable(int)), statusbar_, SLOT(resetCycle()) );
    connect( actionFrame_, SIGNAL(cyclesAvailable(int)), statusbar_, SLOT(setNCycles(int)) );

    // includes
    connect( interface_, SIGNAL(busy()), includeFrame_, SLOT(requestUpdate()) );
    connect( interface_, SIGNAL(includeFilesAvailable(const IncludeFile::List&)), includeFrame_, SLOT(update(const IncludeFile::List&)) );

    // figures
    connect( interface_, SIGNAL(figuresAvailable(const FileRecord::List&)), figureFrame_, SLOT(update(const FileRecord::List&)) );

    // command dialog
    connect( interface_, SIGNAL(commandAdded(Latex::Command)), &commandDialog_->commandModel(), SLOT(add(Latex::Command)) );
    connect( interface_, SIGNAL(commandsAdded(Latex::Command::List)), &commandDialog_->commandModel(), SLOT(add(Latex::Command::List)) );
    connect( interface_, SIGNAL(commandCompleted(Latex::Command)), &commandDialog_->commandModel(), SLOT(remove(Latex::Command)) );

    connect( &buttonFrame_->editButton(), SIGNAL(clicked()), SLOT(editTex()) );
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
    tabWidgetHeight_ = tabWidget_->height();

}

//_____________________________________________________
void MainWindow::setFile( const File& file )
{
    Debug::Throw() << "MainWindow::setFile - file: " << file << endl;
    previousFile_ = File( editor_->text() );
    editor_->setText( file );
}

//_____________________________________
bool MainWindow::checkFile( const File& file, bool quiet )
{ return interface_->checkFile( file, quiet ); }

//_____________________________________
void MainWindow::edit( const QString& extension )
{

    TexFile file( this->file() );
    if( extension.size() ) file = file.addExtension( extension );
    if( _edit( file ) ) setFile( file );

}

//_____________________________________
void MainWindow::open( FileRecord record )
{
    Debug::Throw( "MainWindow::open.\n" );

    if( record.file().isEmpty() )
    {

        // create file dialog
        FileDialog dialog( this );
        if( !file().isEmpty() ) dialog.selectFile( file().path() );
        auto file( dialog.getFile() );
        if( file.isEmpty() ) return;
        else record = FileRecord( file );
    }

    Base::Singleton::get().application<Application>()->addRecentFile( record.file() );
    logFrame_->editor().clear();
    setFile( record.file() );

}

//_______________________________________________
void MainWindow::addLatexHeader()
{

    Debug::Throw( "MainWindow::addLatexHeader.\n" );

    // retrieve input file
    auto file( TexFile( this->file() ).addExtension( ".tex" ) );
    if( !interface_->checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    if( body.contains( "\\begin{document}" ) )
    {
        const auto buffer = QString( "File '%1' already contains a header. <Add Latex Header> canceled." ).arg( file );
        InformationDialog( this, buffer ).exec();
        return;
    }

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( qPrintable( TexString::header() ) );
    out.write( qPrintable( body ) );
    out.write( qPrintable( TexString::trailer() ) );
    out.close();
    return;

}

//_____________________________________
void MainWindow::_processCompletedCommand( const Latex::Command& command )
{

    buttonFrame_->updateButtons();
    switch( command.type() )
    {
        case Latex::Command::Type::Latex:
        case Latex::Command::Type::PDFLatex:
        statusbar_->incrementCycle();
        break;

        default:
        break;
    }
}

//_____________________________________
void MainWindow::_updateFile()
{
    const File file( editor_->text() );
    if( file.truncatedName() != previousFile_.truncatedName() )
    {

        actionFrame_->setFile( file );
        buttonFrame_->setFile( file );
        includeFrame_->setFile( file );
        figureFrame_->setFile( file );

    }

    // update title
    _updateWindowTitle( file );

}

//_______________________________________________
void MainWindow::_toLatexAccents()
{

    Debug::Throw( "MainWindow::_toLatexAccents.\n" );

    // retrieve input file
    auto file( TexFile( this->file() ).addExtension( ".tex" ) );
    if( !interface_->checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    // replace accents
    body = TexString( body ).toLatexAccents();

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( qPrintable( body ) );
    out.close();

    return;

}

//_______________________________________________
void MainWindow::_toTextAccents()
{

    Debug::Throw( "MainWindow::_toTextAccents.\n" );

    // retrieve input file
    File file( TexFile( this->file() ).addExtension( ".tex" ) );
    if( !interface_->checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    // replace accents
    body = TexString( body ).toTextAccents();

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( qPrintable( body ) );
    out.close();

    return;

}

//_______________________________________________
void MainWindow::_updateBibtexFile( const File& file )
{
    Debug::Throw() << "MainWindow::_updateBibtexFile - file: " << file << endl;
    bibtexFile_ = file;
    actionFrame_->updateBibtexFile( file );
}

//____________________________________
void MainWindow::enterEvent( QEvent *event )
{
    Debug::Throw( "MainWindow::enterEvent.\n" );

    // request include files update
    if( includeFrame_->filesModified() )
    { includeFrame_->requestUpdate(); }

    return BaseMainWindow::enterEvent( event );
}

//__________________________________________________________________________
void MainWindow::_showCommandDialog()
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog_->show();
    commandDialog_->raise();

}

//_______________________________________________
void MainWindow::_commandDialogHidden()
{
    Debug::Throw( "Application::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//____________________________________
bool MainWindow::_edit( const File& file )
{

    // check editor
    if( !XmlOptions::get().contains( "TEXT_EDITOR" ) )
    {
        InformationDialog( this, tr( "Text editor not set." ) ).exec();
        return false;
    }

    // check file
    if( !interface_->checkFile( file ) ) return false;

    // update display and run command
    auto application( Base::Singleton::get().application<Application>() );
    application->busy();
    const auto buffer = tr( "Running %1..." ).arg( QString( XmlOptions::get().raw( "TEXT_EDITOR" ) ) );
    statusbar_->label().setText( buffer );
    ( Base::Command( XmlOptions::get().raw( "TEXT_EDITOR" ) ) << file.localName() ).run( file.path() );

    application->idle();
    statusbar_->label().clear();

    return true;
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    BaseMainWindow::closeEvent( event );
    qApp->quit();
}


//_______________________________________
void MainWindow::_updateConfiguration()
{
    Debug::Throw( "MainWindow::_updateConfiguration" );
    actionFrame_->setAutoBibtex( XmlOptions::get().get<bool>( "AUTO_BIBTEX" ) );

    // command dialog
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    else commandDialog_->hide();

}

//_______________________________________
void MainWindow::_installActions()
{

    Debug::Throw( "MainWindow::_installActions.\n" );

    // open
    addAction( openAction_ = new QAction( IconEngine::get( IconNames::Open ), tr( "Open..." ), this ) );
    openAction_->setShortcut( QKeySequence::Open );
    connect( openAction_, SIGNAL(triggered()), SLOT(open()) );

    // convert accents
    addAction( toLatexAccentsAction_ = new QAction( tr( "Text to LaTeX" ), this ) );
    connect( toLatexAccentsAction_, SIGNAL(triggered()), SLOT(_toLatexAccents()) );

    addAction( toTextAccentsAction_ = new QAction( tr( "LaTeX to Text" ), this ) );
    connect( toTextAccentsAction_, SIGNAL(triggered()), SLOT(_toTextAccents()) );

    // show command
    addAction( showCommandDialogAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Command List" ), this ) );
    connect( showCommandDialogAction_, SIGNAL(triggered()), SLOT(_showCommandDialog()) );

}
