#ifndef MainWindow_h
#define MainWindow_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "LineEditor.h"
#include "BaseMainWindow.h"
#include "Counter.h"
#include "Debug.h"
#include "FileRecord.h"
#include "LatexCommand.h"
#include "WindowTitle.h"

#include <QAction>
#include <QEvent>
#include <QTabWidget>

class ActionFrame;
class ButtonFrame;
class FigureFrame;
class IncludeFrame;
class LogFrame;
class MenuBar;
class RecentFilesFrame;
class StatusBar;

namespace Latex
{
    class CommandDialog;
    class Interface;
}


//* main window
class MainWindow: public BaseMainWindow, private Base::Counter<MainWindow>
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor;
    explicit MainWindow( QWidget* );

    //*@name accessors
    //@{

    //* retrieve menubar
    MenuBar &menuBar() const
    { return *menuBar_; }

    //* retrieve ActionFrame
    ActionFrame &actionFrame() const
    { return *actionFrame_; }

    //* retrieve ButtonFrame
    ButtonFrame &buttonFrame() const
    { return *buttonFrame_; }

    //* retrieve LogFrame
    LogFrame &logFrame() const
    { return *logFrame_; }

    //* retrieve IncludeFrame
    IncludeFrame &includeFrame() const
    { return *includeFrame_; }

    //* retrieve FigureFrame
    FigureFrame &figureFrame() const
    { return *figureFrame_; }

    //* retrieve recent file frame
    RecentFilesFrame &recentFilesFrame() const
    { return *recentFilesFrame_; }

    //* retrieve StatusBar
    StatusBar& statusBar() const
    { return *statusbar_; }

    //* latex interface
    Latex::Interface& interface() const
    { return *interface_; }

    //* latex commands
    Latex::CommandDialog& commandDialog() const
    { return *commandDialog_; }

    //* retrieve file from editor
    File file() const
    { return File( editor_->text() ); }

    //* retrieve file from editor
    bool hasBibtexFile() const
    { return !bibtexFile_.isEmpty() && bibtexFile_.exists(); }

    //* bibtex file
    const File& bibtexFile() const
    { return bibtexFile_; }

    //@}

    //*@name modifiers
    //@{

    //* set file
    void setFile( const File& );

    //* check file
    bool checkFile( const File&, bool quiet = false );

    //*@name actions
    //@{

    //* open action
    QAction& openAction() const
    { return *openAction_; }

    //* to latex accent
    QAction& toLatexAccentsAction() const
    { return *toLatexAccentsAction_; }

    //* to text accents
    QAction& toTextAccentsAction() const
    { return *toTextAccentsAction_; }

    //* toggle command frame
    QAction& showCommandDialogAction() const
    { return *showCommandDialogAction_; }

    //@}

    public Q_SLOTS:

    //* edit file (with extension if any, row otherwise)
    void edit( const QString& = QString() );

    //* edit tex file
    void open( FileRecord = FileRecord() );

    //* edit tex file
    void editTex()
    { edit( ".tex" ); }

    //* edit log file
    void editLog()
    { edit( ".log" ); }

    //* edit aux file
    void editAux()
    { edit( ".aux" ); }

    //* edit Toc file
    void editToc()
    { edit( ".toc" ); }

    //* edit Bib file
    void editBib()
    { if( hasBibtexFile() ) _edit( bibtexFile_ ); }

    //* edit Tof file
    void editTof()
    { edit( ".tof" ); }

    //* edit ps file
    void editPS()
    { edit( ".ps" ); }

    //* add latex header/trailer to text file
    void addLatexHeader();

    Q_SIGNALS:

    void resizeRequest();

    protected:

    //* close window event handler
    void closeEvent( QCloseEvent* ) override;

    //* enter
    void enterEvent( QEvent* ) override;

    private Q_SLOTS:

    //* command has been completed
    void _processCompletedCommand( const Latex::Command& );

    //* update file from text editor
    void _updateFile();

    //* update window title
    void _updateWindowTitle( const File& file = File() )
    { setWindowTitle( WindowTitle( file ).get() ); }

    //* replace accents (text to latex)
    void _toLatexAccents();

    //* replace accents (latex to text)
    void _toTextAccents();

    //* bibtex file available
    void _updateBibtexFile( const File& );

    //* toggle command frame visibility
    void _showCommandDialog();

    //* command dialog hidden
    void _commandDialogHidden();

    //* update configuration
    void _updateConfiguration();

    private:

    //* edit
    bool _edit( const File& );

    //* install actions
    void _installActions();

    //* needed to keep track of actual file changes
    File previousFile_;

    //* bibliography file
    File bibtexFile_;

    //* latex interface
    Latex::Interface* interface_ = nullptr;

    //* top widget
    QWidget* top_ = nullptr;

    //* file line editor singleton
    LineEditor* editor_ = nullptr;

    //* action frame singleton
    ActionFrame* actionFrame_ = nullptr;

    //* button frame singleton
    ButtonFrame* buttonFrame_ = nullptr;

    //* command list
    Latex::CommandDialog* commandDialog_ = nullptr;

    //*@name actions
    //@{

    //* open
    QAction* openAction_ = nullptr;

    //* to latex accents
    QAction* toLatexAccentsAction_ = nullptr;

    //* to text accents
    QAction* toTextAccentsAction_ = nullptr;

    //* toggle command frame action
    QAction* showCommandDialogAction_ = nullptr;

    //@}

    //* tab widget
    QTabWidget *tabWidget_ = nullptr;

    //* tab widget size
    int tabWidgetHeight_ = 0;

    //* log frame singleton
    LogFrame* logFrame_ = nullptr;

    //* includes frame singleton
    IncludeFrame* includeFrame_ = nullptr;

    //* figures frame singleton
    FigureFrame* figureFrame_ = nullptr;

    //* recent files frame
    RecentFilesFrame* recentFilesFrame_ = nullptr;

    //* state frame singleton
    StatusBar* statusbar_ = nullptr;

    //* Menu singleton
    MenuBar* menuBar_ = nullptr;

};

#endif
